@@ -17,6 +17,7 @@ from typing import Dict
 
 import torch
 import torch.nn as nn
+from opt_einsum import contract
 
 from .utils import register_grad_sampler
 
@@ -33,9 +34,10 @@ def compute_linear_grad_sample(
         activations: Activations
         backprops: Backpropagations
     """
-    gs = torch.einsum("n...i,n...j->nij", backprops, activations)
-    ret = {layer.weight: gs}
-    if layer.bias is not None:
-        ret[layer.bias] = torch.einsum("n...k->nk", backprops)
-
+    ret = {}
+    if layer.weight.requires_grad:
+        gs = contract("n...i,n...j->nij", backprops, activations)
+        ret[layer.weight] = gs
+    if layer.bias is not None and layer.bias.requires_grad:
+        ret[layer.bias] = contract("n...k->nk", backprops)
     return ret
