@@ -19,7 +19,7 @@ from typing import Callable, List, Optional, Union
 
 import torch
 from opacus.optimizers.utils import params
-from opt_einsum import contract
+from opt_einsum.contract import contract
 from torch import nn
 from torch.optim import Optimizer
 
@@ -113,7 +113,7 @@ def _generate_noise(
         reference: The reference Tensor to get the appropriate shape and device
             for generating the noise
         generator: The PyTorch noise generator
-        secure_mode: boolean showing if "secure" noise need to be generate
+        secure_mode: boolean showing if "secure" noise need to be generated
             (see the notes)
 
     Notes:
@@ -166,47 +166,6 @@ def _generate_noise(
         )
 
 
-def _get_flat_grad_sample(p: torch.Tensor):
-    """
-    Return parameter's per sample gradients as a single tensor.
-
-    By default, per sample gradients (``p.grad_sample``) are stored as one tensor per
-    batch basis. Therefore, ``p.grad_sample`` is a single tensor if holds results from
-    only one batch, and a list of tensors if gradients are accumulated over multiple
-    steps. This is done to provide visibility into which sample belongs to which batch,
-    and how many batches have been processed.
-
-    This method returns per sample gradients as a single concatenated tensor, regardless
-    of how many batches have been accumulated
-
-    Args:
-        p: Parameter tensor. Must have ``grad_sample`` attribute
-
-    Returns:
-        ``p.grad_sample`` if it's a tensor already, or a single tensor computed by
-        concatenating every tensor in ``p.grad_sample`` if it's a list
-
-    Raises:
-        ValueError
-            If ``p`` is missing ``grad_sample`` attribute
-    """
-
-    if not hasattr(p, "grad_sample"):
-        raise ValueError(
-            "Per sample gradient not found. Are you using GradSampleModule?"
-        )
-    if p.grad_sample is None:
-        raise ValueError(
-            "Per sample gradient is not initialized. Not updated in backward pass?"
-        )
-    if isinstance(p.grad_sample, torch.Tensor):
-        return p.grad_sample
-    elif isinstance(p.grad_sample, list):
-        return torch.cat(p.grad_sample, dim=0)
-    else:
-        raise ValueError(f"Unexpected grad_sample type: {type(p.grad_sample)}")
-
-
 class DPOptimizer(Optimizer):
     """
     ``torch.optim.Optimizer`` wrapper that adds additional functionality to clip per
@@ -227,7 +186,7 @@ class DPOptimizer(Optimizer):
     Examples:
         >>> module = MyCustomModel()
         >>> optimizer = torch.optim.SGD(module.parameters(), lr=0.1)
-        >>> dp_optimzer = DPOptimizer(
+        >>> dp_optimizer = DPOptimizer(
         ...     optimizer=optimizer,
         ...     noise_multiplier=1.0,
         ...     max_grad_norm=1.0,
@@ -290,6 +249,48 @@ class DPOptimizer(Optimizer):
         for p in self.params:
             p.summed_grad = None
 
+    def _get_flat_grad_sample(self, p: torch.Tensor):
+        """
+        Return parameter's per sample gradients as a single tensor.
+
+        By default, per sample gradients (``p.grad_sample``) are stored as one tensor per
+        batch basis. Therefore, ``p.grad_sample`` is a single tensor if holds results from
+        only one batch, and a list of tensors if gradients are accumulated over multiple
+        steps. This is done to provide visibility into which sample belongs to which batch,
+        and how many batches have been processed.
+
+        This method returns per sample gradients as a single concatenated tensor, regardless
+        of how many batches have been accumulated
+
+        Args:
+            p: Parameter tensor. Must have ``grad_sample`` attribute
+
+        Returns:
+            ``p.grad_sample`` if it's a tensor already, or a single tensor computed by
+            concatenating every tensor in ``p.grad_sample`` if it's a list
+
+        Raises:
+            ValueError
+                If ``p`` is missing ``grad_sample`` attribute
+        """
+
+        if not hasattr(p, "grad_sample"):
+            raise ValueError(
+                "Per sample gradient not found. Are you using GradSampleModule?"
+            )
+        if p.grad_sample is None:
+            raise ValueError(
+                "Per sample gradient is not initialized. Not updated in backward pass?"
+            )
+        if isinstance(p.grad_sample, torch.Tensor):
+            ret = p.grad_sample
+        elif isinstance(p.grad_sample, list):
+            ret = torch.cat(p.grad_sample, dim=0)
+        else:
+            raise ValueError(f"Unexpected grad_sample type: {type(p.grad_sample)}")
+
+        return ret
+
     def signal_skip_step(self, do_skip=True):
         """
         Signals the optimizer to skip an optimization step and only perform clipping and
@@ -341,7 +342,7 @@ class DPOptimizer(Optimizer):
         """
         ret = []
         for p in self.params:
-            ret.append(_get_flat_grad_sample(p))
+            ret.append(self._get_flat_grad_sample(p))
         return ret
 
     @property
@@ -394,7 +395,7 @@ class DPOptimizer(Optimizer):
         """
 
         per_param_norms = [
-            g.view(len(g), -1).norm(2, dim=-1) for g in self.grad_samples
+            g.reshape(len(g), -1).norm(2, dim=-1) for g in self.grad_samples
         ]
         per_sample_norms = torch.stack(per_param_norms, dim=1).norm(2, dim=1)
         per_sample_clip_factor = (self.max_grad_norm / (per_sample_norms + 1e-6)).clamp(
@@ -403,8 +404,7 @@ class DPOptimizer(Optimizer):
 
         for p in self.params:
             _check_processed_flag(p.grad_sample)
-
-            grad_sample = _get_flat_grad_sample(p)
+            grad_sample = self._get_flat_grad_sample(p)
             grad = contract("i,i...", per_sample_clip_factor, grad_sample)
 
             if p.summed_grad is not None:
@@ -428,7 +428,7 @@ class DPOptimizer(Optimizer):
                 generator=self.generator,
                 secure_mode=self.secure_mode,
             )
-            p.grad = (p.summed_grad + noise).view_as(p.grad)
+            p.grad = (p.summed_grad + noise).view_as(p)
 
             _mark_as_processed(p.summed_grad)
 
@@ -462,7 +462,7 @@ class DPOptimizer(Optimizer):
         """
 
         if set_to_none is False:
-            logger.info(
+            logger.debug(
                 "Despite set_to_none is set to False, "
                 "opacus will set p.grad_sample and p.summed_grad to None due to "
                 "non-trivial gradient accumulation behaviour"
